from typing import Optional

from pydantic import BaseModel, Field


class ExpandInput(BaseModel):
    """Input for the expand operation"""

    key: dict | str
    relation: str


class EdgeKeys(BaseModel):
    start: dict | str
    end: dict | str


class EdgeUserset(BaseModel):
    start: Optional[str] = None
    end: Optional[str] = None


class EdgeInput(BaseModel):
    """RelationKey model."""

    keys: EdgeKeys
    user_set: EdgeUserset = Field(EdgeUserset(), alias="userset")
    relation: str



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.error_model import ErrorModel


T = TypeVar("T", bound="ErrorResponse")


@_attrs_define
class ErrorResponse:
    """Define error response model.

    Attributes:
        error (ErrorModel): ErrorModel class object instance.

    Raises:
        pydantic.error_wrappers.ValidationError: If any of provided attribute
            doesn't pass type validation.

        Attributes:
            error (ErrorModel): Define base error model for the response.

                Attributes:
                    code (int): HTTP error status code.
                    message (str): Detail on HTTP error.
                    status (str): HTTP error reason-phrase as per in RFC7235. NOTE! Set
                        automatically based on HTTP error status code.

                Raises:
                    pydantic.error_wrappers.ValidationError: If any of provided attribute
                        doesn't pass type validation.
    """

    error: "ErrorModel"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        error = self.error.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "error": error,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.error_model import ErrorModel

        d = src_dict.copy()
        error = ErrorModel.from_dict(d.pop("error"))

        error_response = cls(
            error=error,
        )

        error_response.additional_properties = d
        return error_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EdgeUserset")


@_attrs_define
class EdgeUserset:
    """
    Attributes:
        start (Union[Any, Unset, str]):
        end (Union[Any, Unset, str]):
    """

    start: Union[Any, Unset, str] = UNSET
    end: Union[Any, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start: Union[Any, Unset, str]
        if isinstance(self.start, Unset):
            start = UNSET

        else:
            start = self.start

        end: Union[Any, Unset, str]
        if isinstance(self.end, Unset):
            end = UNSET

        else:
            end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_start(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        start = _parse_start(d.pop("start", UNSET))

        def _parse_end(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        end = _parse_end(d.pop("end", UNSET))

        edge_userset = cls(
            start=start,
            end=end,
        )

        edge_userset.additional_properties = d
        return edge_userset

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.ready_response import ReadyResponse
from ...types import Response


def _get_kwargs() -> Dict[str, Any]:
    return {
        "method": "get",
        "url": "/api/ready",
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[ErrorResponse, ReadyResponse]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ReadyResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_GATEWAY:
        response_502 = ErrorResponse.from_dict(response.json())

        return response_502
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ErrorResponse, ReadyResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[ErrorResponse, ReadyResponse]]:
    """Simple health check.

     Run basic application health check.

    If the application is up and running then this endpoint will return simple
    response with status ok. Moreover, if it has Redis enabled then connection
    to it will be tested. If Redis ping fails, then this endpoint will return
    502 HTTP error.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, ReadyResponse]]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[ErrorResponse, ReadyResponse]]:
    """Simple health check.

     Run basic application health check.

    If the application is up and running then this endpoint will return simple
    response with status ok. Moreover, if it has Redis enabled then connection
    to it will be tested. If Redis ping fails, then this endpoint will return
    502 HTTP error.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, ReadyResponse]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[ErrorResponse, ReadyResponse]]:
    """Simple health check.

     Run basic application health check.

    If the application is up and running then this endpoint will return simple
    response with status ok. Moreover, if it has Redis enabled then connection
    to it will be tested. If Redis ping fails, then this endpoint will return
    502 HTTP error.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, ReadyResponse]]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[ErrorResponse, ReadyResponse]]:
    """Simple health check.

     Run basic application health check.

    If the application is up and running then this endpoint will return simple
    response with status ok. Moreover, if it has Redis enabled then connection
    to it will be tested. If Redis ping fails, then this endpoint will return
    502 HTTP error.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, ReadyResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.delete_site_key_type_0 import DeleteSiteKeyType0


T = TypeVar("T", bound="DeleteSite")


@_attrs_define
class DeleteSite:
    """
    Attributes:
        key (Union['DeleteSiteKeyType0', str]):
    """

    key: Union["DeleteSiteKeyType0", str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.delete_site_key_type_0 import DeleteSiteKeyType0

        key: Union[Dict[str, Any], str]

        if isinstance(self.key, DeleteSiteKeyType0):
            key = self.key.to_dict()

        else:
            key = self.key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key": key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.delete_site_key_type_0 import DeleteSiteKeyType0

        d = src_dict.copy()

        def _parse_key(data: object) -> Union["DeleteSiteKeyType0", str]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                key_type_0 = DeleteSiteKeyType0.from_dict(data)

                return key_type_0
            except:  # noqa: E722
                pass
            return cast(Union["DeleteSiteKeyType0", str], data)

        key = _parse_key(d.pop("key"))

        delete_site = cls(
            key=key,
        )

        delete_site.additional_properties = d
        return delete_site

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_site_attributes import CreateSiteAttributes
    from ..models.create_site_key_type_0 import CreateSiteKeyType0


T = TypeVar("T", bound="CreateSite")


@_attrs_define
class CreateSite:
    """
    Attributes:
        key (Union['CreateSiteKeyType0', str]):
        name (str):
        site_id (int):
        attributes (Union[Unset, CreateSiteAttributes]):
        description (Union[Any, Unset, str]):
    """

    key: Union["CreateSiteKeyType0", str]
    name: str
    site_id: int
    attributes: Union[Unset, "CreateSiteAttributes"] = UNSET
    description: Union[Any, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.create_site_key_type_0 import CreateSiteKeyType0

        key: Union[Dict[str, Any], str]

        if isinstance(self.key, CreateSiteKeyType0):
            key = self.key.to_dict()

        else:
            key = self.key

        name = self.name
        site_id = self.site_id
        attributes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes.to_dict()

        description: Union[Any, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET

        else:
            description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key": key,
                "name": name,
                "site_id": site_id,
            }
        )
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_site_attributes import CreateSiteAttributes
        from ..models.create_site_key_type_0 import CreateSiteKeyType0

        d = src_dict.copy()

        def _parse_key(data: object) -> Union["CreateSiteKeyType0", str]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                key_type_0 = CreateSiteKeyType0.from_dict(data)

                return key_type_0
            except:  # noqa: E722
                pass
            return cast(Union["CreateSiteKeyType0", str], data)

        key = _parse_key(d.pop("key"))

        name = d.pop("name")

        site_id = d.pop("site_id")

        _attributes = d.pop("attributes", UNSET)
        attributes: Union[Unset, CreateSiteAttributes]
        if isinstance(_attributes, Unset):
            attributes = UNSET
        else:
            attributes = CreateSiteAttributes.from_dict(_attributes)

        def _parse_description(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        create_site = cls(
            key=key,
            name=name,
            site_id=site_id,
            attributes=attributes,
            description=description,
        )

        create_site.additional_properties = d
        return create_site

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.list_objects_input_key_key import ListObjectsInputKeyKey


T = TypeVar("T", bound="ListObjectsInputKey")


@_attrs_define
class ListObjectsInputKey:
    """Use key to identify user

    Attributes:
        key (ListObjectsInputKeyKey):
        relation (str):
        type (str):
    """

    key: Dict[str, Any]
    relation: str
    type: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.key, Dict):
            key = self.key.to_dict()
        else:
            key = self.key
        relation = self.relation
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key": key,
                "relation": relation,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.list_objects_input_key_key import ListObjectsInputKeyKey

        d = src_dict.copy()
        key = ListObjectsInputKeyKey.from_dict(d.pop("key"))

        relation = d.pop("relation")

        type = d.pop("type")

        list_objects_input_key = cls(
            key=key,
            relation=relation,
            type=type,
        )

        list_objects_input_key.additional_properties = d
        return list_objects_input_key

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.edge_keys_end_type_0 import EdgeKeysEndType0
    from ..models.edge_keys_start_type_0 import EdgeKeysStartType0


T = TypeVar("T", bound="EdgeKeys")


@_attrs_define
class EdgeKeys:
    """
    Attributes:
        start (Union['EdgeKeysStartType0', str]):
        end (Union['EdgeKeysEndType0', str]):
    """

    start: Union["EdgeKeysStartType0", str]
    end: Union["EdgeKeysEndType0", str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.edge_keys_end_type_0 import EdgeKeysEndType0
        from ..models.edge_keys_start_type_0 import EdgeKeysStartType0

        start: Union[Dict[str, Any], str]

        if isinstance(self.start, EdgeKeysStartType0):
            start = self.start.to_dict()

        else:
            start = self.start

        end: Union[Dict[str, Any], str]

        if isinstance(self.end, EdgeKeysEndType0):
            end = self.end.to_dict()

        else:
            end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "start": start,
                "end": end,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.edge_keys_end_type_0 import EdgeKeysEndType0
        from ..models.edge_keys_start_type_0 import EdgeKeysStartType0

        d = src_dict.copy()

        def _parse_start(data: object) -> Union["EdgeKeysStartType0", str]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                start_type_0 = EdgeKeysStartType0.from_dict(data)

                return start_type_0
            except:  # noqa: E722
                pass
            return cast(Union["EdgeKeysStartType0", str], data)

        start = _parse_start(d.pop("start"))

        def _parse_end(data: object) -> Union["EdgeKeysEndType0", str]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                end_type_0 = EdgeKeysEndType0.from_dict(data)

                return end_type_0
            except:  # noqa: E722
                pass
            return cast(Union["EdgeKeysEndType0", str], data)

        end = _parse_end(d.pop("end"))

        edge_keys = cls(
            start=start,
            end=end,
        )

        edge_keys.additional_properties = d
        return edge_keys

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
  from ..models.edge import Edge





T = TypeVar("T", bound="Status")


@_attrs_define
class Status:
    """ Status model.

        Attributes:
            status (bool):
            edge (Union['Edge', Any, Unset]):
     """

    status: bool
    edge: Union['Edge', Any, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.edge import Edge
        status = self.status
        edge: Union[Any, Dict[str, Any], Unset]
        if isinstance(self.edge, Unset):
            edge = UNSET

        elif isinstance(self.edge, Edge):
            edge = UNSET
            if not isinstance(self.edge, Unset):
                edge = self.edge.to_dict()

        else:
            edge = self.edge



        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
        })
        if edge is not UNSET:
            field_dict["edge"] = edge

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.edge import Edge
        d = src_dict.copy()
        status = d.pop("status")

        def _parse_edge(data: object) -> Union['Edge', Any, Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _edge_type_0 = data
                edge_type_0: Union[Unset, Edge]
                if isinstance(_edge_type_0,  Unset):
                    edge_type_0 = UNSET
                else:
                    edge_type_0 = Edge.from_dict(_edge_type_0)



                return edge_type_0
            except: # noqa: E722
                pass
            return cast(Union['Edge', Any, Unset], data)

        edge = _parse_edge(d.pop("edge", UNSET))


        status = cls(
            status=status,
            edge=edge,
        )

        status.additional_properties = d
        return status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.edge_keys import EdgeKeys
    from ..models.edge_userset import EdgeUserset


T = TypeVar("T", bound="EdgeInput")


@_attrs_define
class EdgeInput:
    """RelationKey model.

    Attributes:
        keys (EdgeKeys):
        relation (str):
        userset (Union[Unset, EdgeUserset]):
    """

    keys: "EdgeKeys"
    relation: str
    userset: Union[Unset, "EdgeUserset"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        keys = self.keys.to_dict()

        relation = self.relation
        userset: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.userset, Unset):
            userset = self.userset.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "keys": keys,
                "relation": relation,
            }
        )
        if userset is not UNSET:
            field_dict["userset"] = userset

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.edge_keys import EdgeKeys
        from ..models.edge_userset import EdgeUserset

        d = src_dict.copy()
        keys = EdgeKeys.from_dict(d.pop("keys"))

        relation = d.pop("relation")

        _userset = d.pop("userset", UNSET)
        userset: Union[Unset, EdgeUserset]
        if isinstance(_userset, Unset):
            userset = UNSET
        else:
            userset = EdgeUserset.from_dict(_userset)

        edge_input = cls(
            keys=keys,
            relation=relation,
            userset=userset,
        )

        edge_input.additional_properties = d
        return edge_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import abc
from typing import Dict, Literal, Optional

import devtools as dtz

# from guard_post import Client
from guard_post.api.assigns import (
    assign_edge_api_assigns_add_post as assign_edge,
    check_relationship_api_assigns_check_post as check_relation,
    expand_relationship_api_assigns_expand_post as expand_object,
    list_edges_api_assigns_list_post as list_edges,
    revoke_user_to_org_api_assigns_revoke_post as revoke_edge,
)
from guard_post.api.azure_ad import (
    get_user_api_active_dir_get_user_get as get_user_oauth,
)

# from guard_post.models.http_validation_error import HTTPValidationError
# from guard_post.types import Response
from guard_post.api.orgs import (
    delete_api_orgs_delete_post as delete_orgs,
    list_items_api_orgs_list_get as list_orgs,
    sync_api_orgs_sync_post as sync_orgs,
)
from guard_post.api.sites import (
    delete_api_sites_delete_post as delete_sites,
    list_sites_api_sites_list_get as list_sites,
    sync_api_sites_sync_post as sync_sites,
)
from guard_post.api.users import (
    delete_api_users_delete_post as delete_users,
    list_items_api_users_list_get as list_users,
    sync_api_users_sync_post as sync_users,
)
from guard_post.client import Client
from guard_post.imports import logger

# User, Site, Org
from guard_post.models import (  # CreateSiteAttributes,
    CreateOrg,
    CreateSite,
    CreateUser,
    DeleteOrg,
    DeleteSite,
    DeleteUser,
    EdgeInput,
    ExpandInput,
    ListObjectsInputKey,
    Site as SiteInput,
)

# edge_input,
from guard_post.models.create_user_attributes import CreateUserAttributes
from guard_post.models.edge_keys import EdgeKeys
from guard_post.models.list_objects_input_key_key import ListObjectsInputKeyKey
from guard_post.models.pydantic_models import (
    Organization,
    Organization as OrganizationModel,
    Users,
    Users as UsersModel,
)
from guard_post.models.pydantic_models.relations import (
    EdgeInput as PyEdgeInput,
    EdgeKeys as PyEdgeKeys,
    ExpandInput as PyExpandInput,
)


class Path(abc.ABC):
    def __init__(self, client: "GuardPost") -> None:
        self.client = client

    @abc.abstractmethod
    def get(self, *args, **kwargs) -> None:
        pass

    @abc.abstractmethod
    def list(self, *args, **kwargs) -> None:
        pass

    @abc.abstractmethod
    def create_or_sync(self, *args, **kwargs) -> None:
        pass

    @abc.abstractmethod
    def remove(self, *args, **kwargs) -> None:
        pass

    def class_name(self) -> str:
        return self.__class__.__name__

    def __repr__(self) -> str:
        client_repr = repr(self.client)

        return f"{self.class_name().title()}(client={client_repr[:20]} ...)"


class UserPath(Path):
    def get(self, *args, **kwargs) -> None:
        # Implementation for getting user
        logger.info("Calling organization get")
        # self.client.
        return self
        # logger.warning(f"{} in {self.class_name()}")

    def list(self, limit: int = 100) -> None:
        # Implementation for listing users
        return list_users.sync(client=self.client, limit=limit)

    def create_or_sync(
        self,
        *args,
        key: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        site_id: Optional[int] = None,
        **kwargs,
    ) -> None:
        # Implementation for creating or syncing user
        key_input = key or kwargs.get("key", None) or {"test": "users", "type": "user"}
        # name_input = name or kwargs.get("name", None) or "test_site"
        # description = kwargs.get("description", None) or "test_org_descriptionssssss"
        # site_id = site_id or kwargs.get("site_id", None) or 4
        # logger.info(f"Calling organization create_or_sync with {key_input}")
        create_site = CreateUser(
            key=key_input,
            first_name="test_first_name",
            last_name="test_last_name",
            email=kwargs.get("email", None) or "kevin.hill@hussmann.com",
            phone=kwargs.get("phone", None) or "1234567890",
            attributes=CreateUserAttributes(),
            # name=name_input,
            # description=description,
            # site_id=site_id,
        )
        return sync_users.sync(client=self.client, json_body=create_site)

    # def create_or_sync_object(self, site: )

    def remove(self, key: dict = dict()) -> None:
        # Implementation for removing user
        if not key:
            raise ValueError("key is required")
        del_input = DeleteUser(key=key)
        return delete_users.sync(client=self.client, json_body=del_input)

    def add_groups(self, key: dict, groups: list = []) -> dict:
        # Implementation for adding groups to user
        return {
            "key": key,
            "groups": groups,
            "success": True,
        }

    def add_group(self, key: dict, group: str) -> dict:
        # Implementation for adding group to user
        group_list = [group]
        return self.add_groups(key=key, groups=group_list)

    def remove_groups(self, key: dict, groups: list = []) -> dict:
        # Implementation for removing groups from user
        return {
            "key": key,
            "groups": groups,
            "success": True,
        }

    def remove_group(self, key: dict, group: str) -> dict:
        # Implementation for removing group from user
        group_list = [group]
        return self.remove_groups(key=key, groups=group_list)

    def read_groups(self, key: dict) -> dict:
        # Implementation for reading groups from user
        # Will need to create httpx call to get groups. Make sure it works in the API first
        return {
            "key": key,
            "groups": ["admin", "member"],
            "success": True,
        }

    # def add_roles(self, key: dict, roles: list = []) -> dict:


class SitePath(Path):
    def get(self, *args, **kwargs) -> None:
        # Implementation for getting user
        logger.info("Calling organization get")
        # self.client.
        return self
        # logger.warning(f"{} in {self.class_name()}")

    def list(self, limit: int = 100) -> None:
        # Implementation for listing users
        return list_sites.sync(client=self.client, limit=limit)

    def create_or_sync(
        self,
        *args,
        key: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        site_id: Optional[int] = None,
        **kwargs,
    ) -> None:
        # Implementation for creating or syncing user
        key_input = (
            key or kwargs.get("key", None) or {"test": "organization", "type": "org"}
        )
        name_input = name or kwargs.get("name", None) or "test_site"
        description = kwargs.get("description", None) or "test_org_descriptionssssss"
        site_id = site_id or kwargs.get("site_id", None) or 4
        # logger.info(f"Calling organization create_or_sync with {key_input}")
        create_site = CreateSite(
            key=key_input,
            name=name_input,
            description=description,
            site_id=site_id,
        )
        return sync_sites.sync(client=self.client, json_body=create_site)

    def remove(self, key: dict = dict()) -> None:
        # Implementation for removing user
        if not key:
            raise ValueError("key is required")
        del_input = DeleteSite(key=key)
        return delete_sites.sync(client=self.client, json_body=del_input)


class OrganizationPath(Path):
    def get(self, *args, **kwargs) -> None:
        # Implementation for getting user
        logger.info("Calling organization get")
        # self.client.
        return self
        # logger.warning(f"{} in {self.class_name()}")

    def list(self, limit: int = 100) -> None:
        # Implementation for listing users
        return list_orgs.sync(client=self.client, limit=limit)

    def create_or_sync(
        self,
        *args,
        key: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        **kwargs,
    ) -> None:
        # Implementation for creating or syncing user
        key_input = (
            key or kwargs.get("key", None) or {"test": "organization", "type": "org"}
        )
        name_input = name or kwargs.get("name", None) or "test_org"
        descript = kwargs.get("description", None) or "test_org_descriptionssssss"
        # logger.info(f"Calling organization create_or_sync with {key_input}")
        create_org = CreateOrg(
            key=key_input,
            name=name_input,
            description=descript,
        )
        return sync_orgs.sync(client=self.client, json_body=create_org)

    def remove(self, key: dict = dict(), **kwargs) -> None:
        # Implementation for removing user
        if not key:
            raise ValueError("key is required")
        del_input = DeleteOrg(key=key)
        return delete_orgs.sync(client=self.client, json_body=del_input)


class GuardPost(Client):
    @property
    def users(self) -> "UserPath":
        return UserPath(client=self)

    @property
    def sites(self) -> Path:
        return SitePath(client=self)

    @property
    def orgs(self) -> Path:
        return OrganizationPath(client=self)

    @property
    def httpx_client(self):
        return self.get_httpx_client()

    @property
    def async_httpx_client(self):
        return self.get_async_httpx_client()

    def check(self, source: dict, relation: str, target: dict) -> bool:
        logger.info("Calling check")
        edge_input = EdgeInput(
            keys=EdgeKeys(start=source, end=target), relation=relation
        )
        response = check_relation.sync(client=self, json_body=edge_input)

        return response.get("allowed", False)

    def assign(self, source: dict, relation: str, target: dict) -> bool:
        edge_input = EdgeInput(
            keys=EdgeKeys(start=source, end=target), relation=relation
        )
        assigned = assign_edge.sync(client=self, json_body=edge_input)
        return assigned

    def revoke(self, source: dict, relation: str, target: dict) -> bool:
        edge_input = EdgeInput(
            keys=EdgeKeys(start=source, end=target), relation=relation
        )
        revoked = revoke_edge.sync(client=self, json_body=edge_input)
        return revoked

    def expand(self, source: dict, relation: str) -> bool:
        expand_input = ExpandInput(key=source, relation=relation)
        # logger.info(f"Calling expand with {expand_input}")
        response = expand_object.sync(client=self, json_body=expand_input)
        return response

    @logger.catch
    def resources(self, source: dict, relation: str, relation_type: str) -> bool:
        # expand_input = ExpandInput(key=source, relation=relation)
        list_obj_input = ListObjectsInputKey(
            key=source, relation=relation, type=relation_type
        )
        # logger.info(f"Calling expand with {expand_input}")
        response = list_edges.sync(client=self, json_body=list_obj_input)
        return response

    def oauth_token(self, token: str):
        response = get_user_oauth.sync(
            client=self.with_headers({"Authorization": f"Bearer {token}"})
        )
        return response

    def assign_user_roles(
        self,
        org: OrganizationModel,
        user: UsersModel,
        roles: list[Literal["owner", "admin", "member"]] = [],
    ):
        if not roles:
            raise AttributeError(
                "You need to provide groups/roles for the user and organization"
            )

        target_organization = self.orgs.create_or_sync(**org.get_storable())
        if (
            not isinstance(target_organization, dict)
            or "org" not in target_organization
        ):
            raise ValueError("We weren't able to create or retrieve the organization")
        org_key = target_organization["org"]["key"]

        source_user = self.users.create_or_sync(**user.get_storable())
        if not isinstance(source_user, dict) or "user" not in source_user:
            raise ValueError("Couldn't add or retrieve the user")
        user_key = source_user["user"]["key"]

        if not user_key or not org_key:
            raise AttributeError("We need both a user key and an organization key.")

        assignments = [self.assign(user_key, role, org_key) for role in roles]

        return assignments

    def get_user_roles(self, user: Users, org: Organization):
        endpoint = "/api/assigns/get_relationships"
        edge_keys = PyEdgeKeys(start=user.key, end=org.key)
        edge_input = PyEdgeInput(keys=edge_keys, relation="admin")
        # print(edge_input.model_dump(mode="json", warnings=False))
        response = self.httpx_client.post(endpoint, json=edge_input.model_dump())
        response.raise_for_status()
        resp_json = response.json()
        if "roles" in resp_json:
            return resp_json["roles"]
        return []


def main():
    # /api/active_dir/current_user
    guard = GuardPost(base_url="http://localhost:8000")
    # response = guard.oauth_token(
    #     "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImtpZCI6IjVCM25SeHRRN2ppOGVORGMzRnkwNUtmOTdaRSJ9.eyJhdWQiOiI1YWRmZGFlNC00MDY3LTQ2YTktOWRkNi1lYzdhMjk1ZmM5Y2QiLCJpc3MiOiJodHRwczovL2xvZ2luLm1pY3Jvc29mdG9ubGluZS5jb20vMmU4ZmY3NGQtMDM3Yy00NDA1LWI1MTktOGUwY2IzZGUxZTg1L3YyLjAiLCJpYXQiOjE3MDYyMjc1NTcsIm5iZiI6MTcwNjIyNzU1NywiZXhwIjoxNzA2MjMxNDU3LCJuYW1lIjoiSGlsbCwgS2V2aW4iLCJub25jZSI6ImMwOGE5M2E0LTY0MzgtNDA1Ny04Mzk3LTU1ZjgxMmJhNmU5ZSIsIm9pZCI6IjRkNTcyOWE2LTcxMzMtNDkwNi1iNTBmLTc4NWFlNzVmZjQzZSIsInByZWZlcnJlZF91c2VybmFtZSI6IktldmluLkhpbGxAaHVzc21hbm4uY29tIiwicHJvdl9kYXRhIjpbeyJQcm92IjoiZ2l0aHViLmNvbSIsIkF0Ijp0cnVlLCJBbHRzZWNpZCI6IjI0MDMyNDAiLCJBY2Nlc3NUb2tlbiI6bnVsbH1dLCJyaCI6IjAuQVRjQVRmZVBMbndEQlVTMUdZNE1zOTRlaGVUYTMxcG5RS2xHbmRic2VpbGZ5YzAzQUlJLiIsInN1YiI6IlpKaEZjVzFIcFljblI1cG9ZMTNEUURBcUJ3WVdObi1vZHBxUVpPM3l0OGsiLCJ0aWQiOiIyZThmZjc0ZC0wMzdjLTQ0MDUtYjUxOS04ZTBjYjNkZTFlODUiLCJ1dGkiOiJua2s3M3dxWkZFYXQ1QlhhWkFnY0FRIiwidmVyIjoiMi4wIn0.V01bVZZyFZD1pmNEuo6Tr8xLm0KasdSCHyfodOo8CKh5CfKJD7QffueD9Op3A5i_jpiAyDYlDc9UzDmzp-on8lcLYMKMY4kKwWYpRV4_ucZUWVjWEnsYx0Gfl5kte6XuO01sGAy8fSvVugCn89kE9UxU0g3bVzERJDDTXut5GSlj90SugE3Rqft5mPd-y2IOJEaLsp3ZuZyS8xtdXMs57W67UWmuE0C6dhQsvh5ivuTJE467ky19SUbJnIpYTMicdXiPluMnp6uymJKXTuk0u8Prh-pmgObOCfyNEACNaOJHAH4aGgraad09dPUDs8aOLh5rlvuSiOWsHzwo9YXPww"
    # )
    # dtz.debug(response)
    logger.info("Hello, world!!!!!!!!!!!!!!! I LOVE CANDY!!!!!!!")
    logger.success("I am a success message")
    # logger.debug(guard.users)
    org_key = {"test": "organization", "type": "org", "name": "simple_org"}
    org_key2 = {"test": "organization", "type": "org", "name": "simple_org2"}
    site_key = {"test": "site", "type": "site", "name": "simple_site"}
    user_key = {
        "test": "user",
        "type": "user",
        "name": "simple_user",
        "email": "monika@hussmann.com",
    }
    # logger.debug(site_key)
    # logger.debug(user_key)

    created_org = guard.orgs.create_or_sync(
        key=org_key,
        name="monika's org",
        description="monika's org description",
    )
    logger.warning(created_org)
    created_org2 = guard.orgs.create_or_sync(
        key=org_key2,
        name="kevin's org",
        description="kevin's org description",
    )
    logger.error(created_org2)
    logger.warning(created_org2)
    # dtz.debug(guard.orgs.list(limit=1))
    # # logger.critical(guard.orgs.remove(key=org_key))
    # dtz.debug(guard.orgs.list(limit=1))
    created_site = guard.sites.create_or_sync(
        key=site_key,
        name="monika's site",
        description="monika's site description",
        site_id=44,
    )
    logger.debug(created_site)

    # dtz.debug(guard.sites.list(limit=1))
    # # logger.critical(
    # #     guard.sites.remove(key={"test": "site", "type": "site", "name": "simple_site"})
    # # )

    # dtz.debug(guard.users.list(limit=1))
    created_user = guard.users.create_or_sync(
        key=user_key,
        name="monika's user",
        description="monika's user description",
        phone="+491761234567",
        email="monika@hussmann.com",
    )
    added_groups = guard.users.add_groups(
        key=user_key,
        groups=["admin", "member"],
    )

    logger.warning(created_user)
    # dtz.debug(guard.users.list(limit=1))
    revoked_original = guard.revoke(user_key, "admin", org_key)
    logger.success(revoked_original)
    org_assigned = guard.assign(org_key2, "parent", org_key)
    assigned = guard.assign(user_key, "admin", org_key2)
    logger.success(assigned)
    allowed = guard.check(user_key, "admin", org_key)
    allowed = guard.check(user_key, "can_create_org", org_key)
    allowed = guard.check(user_key, "can_add_member", org_key)
    logger.debug(allowed)
    assert allowed, "check failed"

    expanded = guard.expand(org_key2, "admin")
    logger.info(expanded)
    resources = guard.resources(user_key, "member", "org")
    dtz.debug(resources)


if __name__ == "__main__":
    main()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET

if TYPE_CHECKING:
    from ..models.error_model_details_type_0_item import ErrorModelDetailsType0Item


T = TypeVar("T", bound="ErrorModel")


@_attrs_define
class ErrorModel:
    """Define base error model for the response.

    Attributes:
        code (int): HTTP error status code.
        message (str): Detail on HTTP error.
        status (str): HTTP error reason-phrase as per in RFC7235. NOTE! Set
            automatically based on HTTP error status code.

    Raises:
        pydantic.error_wrappers.ValidationError: If any of provided attribute
            doesn't pass type validation.

        Attributes:
            code (int):
            message (str):
            details (Union[Any, List['ErrorModelDetailsType0Item']]):
    """

    code: int
    message: str
    details: Union[Any, List["ErrorModelDetailsType0Item"]]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        message = self.message
        details: Union[Any, List[Dict[str, Any]]]

        if isinstance(self.details, list):
            details = []
            for details_type_0_item_data in self.details:
                details_type_0_item = details_type_0_item_data.to_dict()

                details.append(details_type_0_item)

        else:
            details = self.details

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "code": code,
                "message": message,
                "details": details,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.error_model_details_type_0_item import ErrorModelDetailsType0Item

        d = src_dict.copy()
        code = d.pop("code")

        message = d.pop("message")

        def _parse_details(
            data: object,
        ) -> Union[Any, List["ErrorModelDetailsType0Item"]]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                details_type_0 = UNSET
                _details_type_0 = data
                for details_type_0_item_data in _details_type_0:
                    details_type_0_item = ErrorModelDetailsType0Item.from_dict(
                        details_type_0_item_data
                    )

                    details_type_0.append(details_type_0_item)

                return details_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Any, List["ErrorModelDetailsType0Item"]], data)

        details = _parse_details(d.pop("details"))

        error_model = cls(
            code=code,
            message=message,
            details=details,
        )

        error_model.additional_properties = d
        return error_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from pydantic import BaseModel, model_validator

# Generic Models

## Node Model

## Relationship Model

# Objects

## User model

## Site model

## Organization model

# Relationship model

## Groups model

# Will have the ability to add a user to a group. We can set application (organization with application).

## Global Group Model

# Inherit from the group model. This will be a group that is global to the application.


## Organization User Model

## Organization Site Model



""" Contains all the data models used in inputs/outputs """

from .azure_user import AzureUser
from .azure_user_acr_type_0 import AzureUserAcrType0
from .azure_user_appidacr_type_0 import AzureUserAppidacrType0
from .azure_user_azpacr_type_0 import AzureUserAzpacrType0
from .azure_user_claims import AzureUserClaims
from .azure_user_ver import AzureUserVer
from .create_org import CreateOrg
from .create_org_attributes import CreateOrgAttributes
from .create_org_key_type_0 import CreateOrgKeyType0
from .create_site import CreateSite
from .create_site_attributes import CreateSiteAttributes
from .create_site_key_type_0 import CreateSiteKeyType0
from .create_user import CreateUser
from .create_user_attributes import CreateUserAttributes
from .create_user_key_type_0 import CreateUserKeyType0
from .delete_org import DeleteOrg
from .delete_org_key_type_0 import DeleteOrgKeyType0
from .delete_site import DeleteSite
from .delete_site_key_type_0 import DeleteSiteKeyType0
from .delete_user import DeleteUser
from .delete_user_key_type_0 import DeleteUserKeyType0
from .edge import Edge
from .edge_input import EdgeInput
from .edge_keys import EdgeKeys
from .edge_keys_end_type_0 import EdgeKeysEndType0
from .edge_keys_start_type_0 import EdgeKeysStartType0
from .edge_userset import EdgeUserset
from .error_model import ErrorModel
from .error_model_details_type_0_item import ErrorModelDetailsType0Item
from .error_response import ErrorResponse
from .expand_input import ExpandInput
from .expand_input_key_type_0 import ExpandInputKeyType0
from .http_validation_error import HTTPValidationError
from .list_objects_input_key import ListObjectsInputKey
from .list_objects_input_key_key import ListObjectsInputKeyKey
from .org import Org
from .org_attributes import OrgAttributes
from .org_key_type_0 import OrgKeyType0
from .ready_response import ReadyResponse
from .site import Site
from .site_attributes import SiteAttributes
from .site_key_type_0 import SiteKeyType0
from .status import Status
from .user import User
from .user_attributes import UserAttributes
from .user_key_type_0 import UserKeyType0
from .validation_error import ValidationError

__all__ = (
    "AzureUser",
    "AzureUserAcrType0",
    "AzureUserAppidacrType0",
    "AzureUserAzpacrType0",
    "AzureUserClaims",
    "AzureUserVer",
    "CreateOrg",
    "CreateOrgAttributes",
    "CreateOrgKeyType0",
    "CreateSite",
    "CreateSiteAttributes",
    "CreateSiteKeyType0",
    "CreateUser",
    "CreateUserAttributes",
    "CreateUserKeyType0",
    "DeleteOrg",
    "DeleteOrgKeyType0",
    "DeleteSite",
    "DeleteSiteKeyType0",
    "DeleteUser",
    "DeleteUserKeyType0",
    "Edge",
    "EdgeInput",
    "EdgeKeys",
    "EdgeKeysEndType0",
    "EdgeKeysStartType0",
    "EdgeUserset",
    "ErrorModel",
    "ErrorModelDetailsType0Item",
    "ErrorResponse",
    "ExpandInput",
    "ExpandInputKeyType0",
    "HTTPValidationError",
    "ListObjectsInputKey",
    "ListObjectsInputKeyKey",
    "Org",
    "OrgAttributes",
    "OrgKeyType0",
    "ReadyResponse",
    "Site",
    "SiteAttributes",
    "SiteKeyType0",
    "Status",
    "User",
    "UserAttributes",
    "UserKeyType0",
    "ValidationError",
)



from pydantic import BaseModel, ConfigDict, Field, PrivateAttr, computed_field


class Site(BaseModel):
    name: str
    description: str | None = None
    site_id: int

    @computed_field
    def key(self) -> dict:
        return {"site_id": self.site_id, "type": "site"}

    def get_storable(self):
        item = self.model_dump(mode="json")
        key = item.pop("key", None)
        if not key:
            item["key"] = {"site_id": item["site_id"], "type": "site"}
            return item
        if isinstance(key, dict):
            key["type"] = "site"
            key["site_id"] = item["site_id"]
            item["key"] = key
        return item


class Users(BaseModel):
    key: dict | str
    first_name: str
    last_name: str
    email: str
    phone: str | None
    attributes: dict

    def get_storable(self):
        item = self.model_dump(mode="json")
        key = item.pop("key", None)
        if not key:
            item["key"] = {"email": item["email"], "type": "user"}
            return item
        if isinstance(key, dict):
            if "test" in key:
                key.pop("test", None)
            key["type"] = "user"
            key["email"] = item["email"]
            item["key"] = key
        return item


class Organization(BaseModel):
    org_id: int | None = Field(None, alias="ID")
    # key: dict | str | None = None
    name: str
    description: str | None = None
    attributes: dict = {}
    model_config: ConfigDict = ConfigDict(cache_strings="all")

    @computed_field
    @property
    def key(self) -> dict:
        return {"name": self.name, "type": "org"}

    def get_storable(self):
        item = self.model_dump(mode="json")
        key = item.pop("key", None)
        if not key:
            item["key"] = {"name": self.name, "type": "org"}
            return item
        if isinstance(key, dict):
            if "type" not in key:
                key["type"] = "org"
            if "name" not in key:
                key["name"] = self.name
            item["key"] = key
        return item

    # groups: list[str] = ["ADMIN", "GUEST"]



from datetime import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from pydantic import BaseModel, EmailStr, Field

from ..models.azure_user_acr_type_0 import AzureUserAcrType0
from ..models.azure_user_appidacr_type_0 import AzureUserAppidacrType0
from ..models.azure_user_azpacr_type_0 import AzureUserAzpacrType0
from ..models.azure_user_ver import AzureUserVer
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.azure_user_claims import AzureUserClaims


T = TypeVar("T", bound="AzureUser")


class User(BaseModel):
    email: EmailStr = Field(..., example="example.name@hussmann.com")
    expiration: datetime = Field(..., example="2024-01-26T01:10:57Z")
    first: str = Field(..., example="Example")
    issued: datetime = Field(..., example="2024-01-26T00:05:57Z")
    last: str = Field(..., example="Name")


@_attrs_define
class AzureUser:
    """
    Attributes:
        aud (str): Identifies the intended audience of the token. In v2.0 tokens, this value is always the client ID of
            the API. In v1.0 tokens, it can be the client ID or the resource URI used in the request.
        iss (str): Identifies the STS that constructs and returns the token, and the Azure AD tenant of the
            authenticated user. If the token issued is a v2.0 token (see the ver claim), the URI ends in /v2.0.
        iat (int): Specifies when the authentication for this token occurred.
        nbf (int): Specifies the time after which the JWT can be processed.
        exp (int): Specifies the expiration time before which the JWT can be accepted for processing.
        sub (str): The principal associated with the token.
        ver (AzureUserVer): Indicates the version of the access token.
        claims (AzureUserClaims): The entire decoded token
        access_token (str): The access_token. Can be used for fetching the Graph API
        idp (Union[Any, Unset, str]): Records the identity provider that authenticated the subject of the token. This
            value is identical to the value of the Issuer claim unless the user account is not in the same tenant as the
            issuer, such as guests. Use the value of iss if the claim is not present.
        aio (Union[Any, Unset, str]): An internal claim used by Azure AD to record data for token reuse. Resources
            should not use this claim.
        name (Union[Any, Unset, str]): Provides a human-readable value that identifies the subject of the token.
        scp (Union[Unset, List[str]]): The set of scopes exposed by the application for which the client application has
            requested (and received) consent. Only included for user tokens.
        roles (Union[Unset, List[str]]): The set of permissions exposed by the application that the requesting
            application or user has been given permission to call.
        wids (Union[Unset, List[str]]): Denotes the tenant-wide roles assigned to this user, from the section of roles
            present in Azure AD built-in roles.
        groups (Union[Unset, List[str]]): Provides object IDs that represent the group memberships of the subject.
        oid (Union[Any, Unset, str]): The immutable identifier for the requestor, which is the verified identity of the
            user or service principal
        tid (Union[Any, Unset, str]): Represents the tenant that the user is signing in to
        uti (Union[Any, Unset, str]): Token identifier claim, equivalent to jti in the JWT specification. Unique, per-
            token identifier that is case-sensitive.
        rh (Union[Any, Unset, str]): Token identifier claim, equivalent to jti in the JWT specification. Unique, per-
            token identifier that is case-sensitive.
        acct (Union[Any, Unset, str]): User's account status in tenant
        auth_time (Union[Any, Unset, int]): Time when the user last authenticated; See OpenID Connect spec
        ctry (Union[Any, Unset, str]): User's country/region
        email (Union[Any, Unset, str]): The addressable email for this user, if the user has one
        family_name (Union[Any, Unset, str]): Provides the last name, surname, or family name of the user as defined in
            the user object
        fwd (Union[Any, Unset, str]): IP address
        given_name (Union[Any, Unset, str]): Provides the first or "given" name of the user, as set on the user object
        idtyp (Union[Any, Unset, str]): Signals whether the token is an app-only token
        in_corp (Union[Any, Unset, str]): Signals if the client is logging in from the corporate network; if they are
            not, the claim is not included
        ipaddr (Union[Any, Unset, str]): The IP address the user authenticated from.
        login_hint (Union[Any, Unset, str]): Login hint
        onprem_sid (Union[Any, Unset, str]): On-premises security identifier
        pwd_exp (Union[Any, Unset, str]): The datetime at which the password expires
        pwd_url (Union[Any, Unset, str]): A URL that the user can visit to change their password
        sid (Union[Any, Unset, str]): Session ID, used for per-session user sign out
        tenant_ctry (Union[Any, Unset, str]): Resource tenant's country/region
        tenant_region_scope (Union[Any, Unset, str]): Region of the resource tenant
        upn (Union[Any, Unset, str]): An identifier for the user that can be used with the username_hint parameter; not
            a durable identifier for the user and should not be used to key data
        verified_primary_email (Union[Unset, List[str]]): Sourced from the user's PrimaryAuthoritativeEmail
        verified_secondary_email (Union[Unset, List[str]]): Sourced from the user's SecondaryAuthoritativeEmail
        vnet (Union[Any, Unset, str]): VNET specifier information
        xms_pdl (Union[Any, Unset, str]): Preferred data location
        xms_pl (Union[Any, Unset, str]): User-preferred language
        xms_tpl (Union[Any, Unset, str]): Tenant-preferred language
        ztdid (Union[Any, Unset, str]): Zero-touch Deployment ID
        acr (Union[Any, AzureUserAcrType0, Unset]): A value of 0 for the "Authentication context class" claim indicates
            the end-user authentication did not meet the requirements of ISO/IEC 29115. Only available in V1.0 tokens
        amr (Union[Unset, List[str]]): Identifies the authentication method of the subject of the token. Only available
            in V1.0 tokens
        appid (Union[Any, Unset, str]): The application ID of the client using the token. Only available in V1.0 tokens
        appidacr (Union[Any, AzureUserAppidacrType0, Unset]): Indicates authentication method of the client. Only
            available in V1.0 tokens
        unique_name (Union[Any, Unset, str]): Provides a human readable value that identifies the subject of the token.
            Only available in V1.0 tokens
        azp (Union[Any, Unset, str]): The application ID of the client using the token. Only available in V2.0 tokens
        azpacr (Union[Any, AzureUserAzpacrType0, Unset]): Indicates the authentication method of the client. Only
            available in V2.0 tokens
        preferred_username (Union[Any, Unset, str]): The primary username that represents the user. Only available in
            V2.0 tokens
        is_guest (Union[Unset, bool]): The user is a guest user in the tenant
    """

    aud: str
    iss: str
    iat: int
    nbf: int
    exp: int
    sub: str
    ver: AzureUserVer
    claims: "AzureUserClaims"
    access_token: str
    idp: Union[Any, Unset, str] = UNSET
    aio: Union[Any, Unset, str] = UNSET
    name: Union[Any, Unset, str] = UNSET
    scp: Union[Unset, List[str]] = UNSET
    roles: Union[Unset, List[str]] = UNSET
    wids: Union[Unset, List[str]] = UNSET
    groups: Union[Unset, List[str]] = UNSET
    oid: Union[Any, Unset, str] = UNSET
    tid: Union[Any, Unset, str] = UNSET
    uti: Union[Any, Unset, str] = UNSET
    rh: Union[Any, Unset, str] = UNSET
    acct: Union[Any, Unset, str] = UNSET
    auth_time: Union[Any, Unset, int] = UNSET
    ctry: Union[Any, Unset, str] = UNSET
    email: Union[Any, Unset, str] = UNSET
    family_name: Union[Any, Unset, str] = UNSET
    fwd: Union[Any, Unset, str] = UNSET
    given_name: Union[Any, Unset, str] = UNSET
    idtyp: Union[Any, Unset, str] = UNSET
    in_corp: Union[Any, Unset, str] = UNSET
    ipaddr: Union[Any, Unset, str] = UNSET
    login_hint: Union[Any, Unset, str] = UNSET
    onprem_sid: Union[Any, Unset, str] = UNSET
    pwd_exp: Union[Any, Unset, str] = UNSET
    pwd_url: Union[Any, Unset, str] = UNSET
    sid: Union[Any, Unset, str] = UNSET
    tenant_ctry: Union[Any, Unset, str] = UNSET
    tenant_region_scope: Union[Any, Unset, str] = UNSET
    upn: Union[Any, Unset, str] = UNSET
    verified_primary_email: Union[Unset, List[str]] = UNSET
    verified_secondary_email: Union[Unset, List[str]] = UNSET
    vnet: Union[Any, Unset, str] = UNSET
    xms_pdl: Union[Any, Unset, str] = UNSET
    xms_pl: Union[Any, Unset, str] = UNSET
    xms_tpl: Union[Any, Unset, str] = UNSET
    ztdid: Union[Any, Unset, str] = UNSET
    acr: Union[Any, AzureUserAcrType0, Unset] = UNSET
    amr: Union[Unset, List[str]] = UNSET
    appid: Union[Any, Unset, str] = UNSET
    appidacr: Union[Any, AzureUserAppidacrType0, Unset] = UNSET
    unique_name: Union[Any, Unset, str] = UNSET
    azp: Union[Any, Unset, str] = UNSET
    azpacr: Union[Any, AzureUserAzpacrType0, Unset] = UNSET
    preferred_username: Union[Any, Unset, str] = UNSET
    is_guest: Union[Unset, bool] = False
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        aud = self.aud
        iss = self.iss
        iat = self.iat
        nbf = self.nbf
        exp = self.exp
        sub = self.sub
        ver = self.ver.value

        claims = self.claims.to_dict()

        access_token = self.access_token
        idp: Union[Any, Unset, str]
        if isinstance(self.idp, Unset):
            idp = UNSET

        else:
            idp = self.idp

        aio: Union[Any, Unset, str]
        if isinstance(self.aio, Unset):
            aio = UNSET

        else:
            aio = self.aio

        name: Union[Any, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET

        else:
            name = self.name

        scp: Union[Unset, List[str]] = UNSET
        if not isinstance(self.scp, Unset):
            scp = self.scp

        roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = self.roles

        wids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.wids, Unset):
            wids = self.wids

        groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = self.groups

        oid: Union[Any, Unset, str]
        if isinstance(self.oid, Unset):
            oid = UNSET

        else:
            oid = self.oid

        tid: Union[Any, Unset, str]
        if isinstance(self.tid, Unset):
            tid = UNSET

        else:
            tid = self.tid

        uti: Union[Any, Unset, str]
        if isinstance(self.uti, Unset):
            uti = UNSET

        else:
            uti = self.uti

        rh: Union[Any, Unset, str]
        if isinstance(self.rh, Unset):
            rh = UNSET

        else:
            rh = self.rh

        acct: Union[Any, Unset, str]
        if isinstance(self.acct, Unset):
            acct = UNSET

        else:
            acct = self.acct

        auth_time: Union[Any, Unset, int]
        if isinstance(self.auth_time, Unset):
            auth_time = UNSET

        else:
            auth_time = self.auth_time

        ctry: Union[Any, Unset, str]
        if isinstance(self.ctry, Unset):
            ctry = UNSET

        else:
            ctry = self.ctry

        email: Union[Any, Unset, str]
        if isinstance(self.email, Unset):
            email = UNSET

        else:
            email = self.email

        family_name: Union[Any, Unset, str]
        if isinstance(self.family_name, Unset):
            family_name = UNSET

        else:
            family_name = self.family_name

        fwd: Union[Any, Unset, str]
        if isinstance(self.fwd, Unset):
            fwd = UNSET

        else:
            fwd = self.fwd

        given_name: Union[Any, Unset, str]
        if isinstance(self.given_name, Unset):
            given_name = UNSET

        else:
            given_name = self.given_name

        idtyp: Union[Any, Unset, str]
        if isinstance(self.idtyp, Unset):
            idtyp = UNSET

        else:
            idtyp = self.idtyp

        in_corp: Union[Any, Unset, str]
        if isinstance(self.in_corp, Unset):
            in_corp = UNSET

        else:
            in_corp = self.in_corp

        ipaddr: Union[Any, Unset, str]
        if isinstance(self.ipaddr, Unset):
            ipaddr = UNSET

        else:
            ipaddr = self.ipaddr

        login_hint: Union[Any, Unset, str]
        if isinstance(self.login_hint, Unset):
            login_hint = UNSET

        else:
            login_hint = self.login_hint

        onprem_sid: Union[Any, Unset, str]
        if isinstance(self.onprem_sid, Unset):
            onprem_sid = UNSET

        else:
            onprem_sid = self.onprem_sid

        pwd_exp: Union[Any, Unset, str]
        if isinstance(self.pwd_exp, Unset):
            pwd_exp = UNSET

        else:
            pwd_exp = self.pwd_exp

        pwd_url: Union[Any, Unset, str]
        if isinstance(self.pwd_url, Unset):
            pwd_url = UNSET

        else:
            pwd_url = self.pwd_url

        sid: Union[Any, Unset, str]
        if isinstance(self.sid, Unset):
            sid = UNSET

        else:
            sid = self.sid

        tenant_ctry: Union[Any, Unset, str]
        if isinstance(self.tenant_ctry, Unset):
            tenant_ctry = UNSET

        else:
            tenant_ctry = self.tenant_ctry

        tenant_region_scope: Union[Any, Unset, str]
        if isinstance(self.tenant_region_scope, Unset):
            tenant_region_scope = UNSET

        else:
            tenant_region_scope = self.tenant_region_scope

        upn: Union[Any, Unset, str]
        if isinstance(self.upn, Unset):
            upn = UNSET

        else:
            upn = self.upn

        verified_primary_email: Union[Unset, List[str]] = UNSET
        if not isinstance(self.verified_primary_email, Unset):
            verified_primary_email = self.verified_primary_email

        verified_secondary_email: Union[Unset, List[str]] = UNSET
        if not isinstance(self.verified_secondary_email, Unset):
            verified_secondary_email = self.verified_secondary_email

        vnet: Union[Any, Unset, str]
        if isinstance(self.vnet, Unset):
            vnet = UNSET

        else:
            vnet = self.vnet

        xms_pdl: Union[Any, Unset, str]
        if isinstance(self.xms_pdl, Unset):
            xms_pdl = UNSET

        else:
            xms_pdl = self.xms_pdl

        xms_pl: Union[Any, Unset, str]
        if isinstance(self.xms_pl, Unset):
            xms_pl = UNSET

        else:
            xms_pl = self.xms_pl

        xms_tpl: Union[Any, Unset, str]
        if isinstance(self.xms_tpl, Unset):
            xms_tpl = UNSET

        else:
            xms_tpl = self.xms_tpl

        ztdid: Union[Any, Unset, str]
        if isinstance(self.ztdid, Unset):
            ztdid = UNSET

        else:
            ztdid = self.ztdid

        acr: Union[Any, Unset, str]
        if isinstance(self.acr, Unset):
            acr = UNSET

        elif isinstance(self.acr, AzureUserAcrType0):
            acr = UNSET
            if not isinstance(self.acr, Unset):
                acr = self.acr.value

        else:
            acr = self.acr

        amr: Union[Unset, List[str]] = UNSET
        if not isinstance(self.amr, Unset):
            amr = self.amr

        appid: Union[Any, Unset, str]
        if isinstance(self.appid, Unset):
            appid = UNSET

        else:
            appid = self.appid

        appidacr: Union[Any, Unset, str]
        if isinstance(self.appidacr, Unset):
            appidacr = UNSET

        elif isinstance(self.appidacr, AzureUserAppidacrType0):
            appidacr = UNSET
            if not isinstance(self.appidacr, Unset):
                appidacr = self.appidacr.value

        else:
            appidacr = self.appidacr

        unique_name: Union[Any, Unset, str]
        if isinstance(self.unique_name, Unset):
            unique_name = UNSET

        else:
            unique_name = self.unique_name

        azp: Union[Any, Unset, str]
        if isinstance(self.azp, Unset):
            azp = UNSET

        else:
            azp = self.azp

        azpacr: Union[Any, Unset, str]
        if isinstance(self.azpacr, Unset):
            azpacr = UNSET

        elif isinstance(self.azpacr, AzureUserAzpacrType0):
            azpacr = UNSET
            if not isinstance(self.azpacr, Unset):
                azpacr = self.azpacr.value

        else:
            azpacr = self.azpacr

        preferred_username: Union[Any, Unset, str]
        if isinstance(self.preferred_username, Unset):
            preferred_username = UNSET

        else:
            preferred_username = self.preferred_username

        is_guest = self.is_guest

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "aud": aud,
                "iss": iss,
                "iat": iat,
                "nbf": nbf,
                "exp": exp,
                "sub": sub,
                "ver": ver,
                "claims": claims,
                "access_token": access_token,
            }
        )
        if idp is not UNSET:
            field_dict["idp"] = idp
        if aio is not UNSET:
            field_dict["aio"] = aio
        if name is not UNSET:
            field_dict["name"] = name
        if scp is not UNSET:
            field_dict["scp"] = scp
        if roles is not UNSET:
            field_dict["roles"] = roles
        if wids is not UNSET:
            field_dict["wids"] = wids
        if groups is not UNSET:
            field_dict["groups"] = groups
        if oid is not UNSET:
            field_dict["oid"] = oid
        if tid is not UNSET:
            field_dict["tid"] = tid
        if uti is not UNSET:
            field_dict["uti"] = uti
        if rh is not UNSET:
            field_dict["rh"] = rh
        if acct is not UNSET:
            field_dict["acct"] = acct
        if auth_time is not UNSET:
            field_dict["auth_time"] = auth_time
        if ctry is not UNSET:
            field_dict["ctry"] = ctry
        if email is not UNSET:
            field_dict["email"] = email
        if family_name is not UNSET:
            field_dict["family_name"] = family_name
        if fwd is not UNSET:
            field_dict["fwd"] = fwd
        if given_name is not UNSET:
            field_dict["given_name"] = given_name
        if idtyp is not UNSET:
            field_dict["idtyp"] = idtyp
        if in_corp is not UNSET:
            field_dict["in_corp"] = in_corp
        if ipaddr is not UNSET:
            field_dict["ipaddr"] = ipaddr
        if login_hint is not UNSET:
            field_dict["login_hint"] = login_hint
        if onprem_sid is not UNSET:
            field_dict["onprem_sid"] = onprem_sid
        if pwd_exp is not UNSET:
            field_dict["pwd_exp"] = pwd_exp
        if pwd_url is not UNSET:
            field_dict["pwd_url"] = pwd_url
        if sid is not UNSET:
            field_dict["sid"] = sid
        if tenant_ctry is not UNSET:
            field_dict["tenant_ctry"] = tenant_ctry
        if tenant_region_scope is not UNSET:
            field_dict["tenant_region_scope"] = tenant_region_scope
        if upn is not UNSET:
            field_dict["upn"] = upn
        if verified_primary_email is not UNSET:
            field_dict["verified_primary_email"] = verified_primary_email
        if verified_secondary_email is not UNSET:
            field_dict["verified_secondary_email"] = verified_secondary_email
        if vnet is not UNSET:
            field_dict["vnet"] = vnet
        if xms_pdl is not UNSET:
            field_dict["xms_pdl"] = xms_pdl
        if xms_pl is not UNSET:
            field_dict["xms_pl"] = xms_pl
        if xms_tpl is not UNSET:
            field_dict["xms_tpl"] = xms_tpl
        if ztdid is not UNSET:
            field_dict["ztdid"] = ztdid
        if acr is not UNSET:
            field_dict["acr"] = acr
        if amr is not UNSET:
            field_dict["amr"] = amr
        if appid is not UNSET:
            field_dict["appid"] = appid
        if appidacr is not UNSET:
            field_dict["appidacr"] = appidacr
        if unique_name is not UNSET:
            field_dict["unique_name"] = unique_name
        if azp is not UNSET:
            field_dict["azp"] = azp
        if azpacr is not UNSET:
            field_dict["azpacr"] = azpacr
        if preferred_username is not UNSET:
            field_dict["preferred_username"] = preferred_username
        if is_guest is not UNSET:
            field_dict["is_guest"] = is_guest

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.azure_user_claims import AzureUserClaims

        d = src_dict.copy()
        aud = d.pop("aud", None)

        iss = d.pop("iss", None)

        iat = d.pop("iat", None)

        nbf = d.pop("nbf", None)

        exp = d.pop("exp", None)

        sub = d.pop("sub", None)

        ver = AzureUserVer(d.pop("ver"))

        claims = AzureUserClaims.from_dict(d.pop("claims"))

        access_token = d.pop("access_token")

        def _parse_idp(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        idp = _parse_idp(d.pop("idp", UNSET))

        def _parse_aio(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        aio = _parse_aio(d.pop("aio", UNSET))

        def _parse_name(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        scp = cast(List[str], d.pop("scp", UNSET))

        roles = cast(List[str], d.pop("roles", UNSET))

        wids = cast(List[str], d.pop("wids", UNSET))

        groups = cast(List[str], d.pop("groups", UNSET))

        def _parse_oid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        oid = _parse_oid(d.pop("oid", UNSET))

        def _parse_tid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        tid = _parse_tid(d.pop("tid", UNSET))

        def _parse_uti(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        uti = _parse_uti(d.pop("uti", UNSET))

        def _parse_rh(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        rh = _parse_rh(d.pop("rh", UNSET))

        def _parse_acct(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        acct = _parse_acct(d.pop("acct", UNSET))

        def _parse_auth_time(data: object) -> Union[Any, Unset, int]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, int], data)

        auth_time = _parse_auth_time(d.pop("auth_time", UNSET))

        def _parse_ctry(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        ctry = _parse_ctry(d.pop("ctry", UNSET))

        def _parse_email(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        email = _parse_email(d.pop("email", UNSET))

        def _parse_family_name(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        family_name = _parse_family_name(d.pop("family_name", UNSET))

        def _parse_fwd(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        fwd = _parse_fwd(d.pop("fwd", UNSET))

        def _parse_given_name(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        given_name = _parse_given_name(d.pop("given_name", UNSET))

        def _parse_idtyp(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        idtyp = _parse_idtyp(d.pop("idtyp", UNSET))

        def _parse_in_corp(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        in_corp = _parse_in_corp(d.pop("in_corp", UNSET))

        def _parse_ipaddr(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        ipaddr = _parse_ipaddr(d.pop("ipaddr", UNSET))

        def _parse_login_hint(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        login_hint = _parse_login_hint(d.pop("login_hint", UNSET))

        def _parse_onprem_sid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        onprem_sid = _parse_onprem_sid(d.pop("onprem_sid", UNSET))

        def _parse_pwd_exp(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        pwd_exp = _parse_pwd_exp(d.pop("pwd_exp", UNSET))

        def _parse_pwd_url(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        pwd_url = _parse_pwd_url(d.pop("pwd_url", UNSET))

        def _parse_sid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        sid = _parse_sid(d.pop("sid", UNSET))

        def _parse_tenant_ctry(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        tenant_ctry = _parse_tenant_ctry(d.pop("tenant_ctry", UNSET))

        def _parse_tenant_region_scope(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        tenant_region_scope = _parse_tenant_region_scope(
            d.pop("tenant_region_scope", UNSET)
        )

        def _parse_upn(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        upn = _parse_upn(d.pop("upn", UNSET))

        verified_primary_email = cast(List[str], d.pop("verified_primary_email", UNSET))

        verified_secondary_email = cast(
            List[str], d.pop("verified_secondary_email", UNSET)
        )

        def _parse_vnet(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        vnet = _parse_vnet(d.pop("vnet", UNSET))

        def _parse_xms_pdl(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        xms_pdl = _parse_xms_pdl(d.pop("xms_pdl", UNSET))

        def _parse_xms_pl(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        xms_pl = _parse_xms_pl(d.pop("xms_pl", UNSET))

        def _parse_xms_tpl(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        xms_tpl = _parse_xms_tpl(d.pop("xms_tpl", UNSET))

        def _parse_ztdid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        ztdid = _parse_ztdid(d.pop("ztdid", UNSET))

        def _parse_acr(data: object) -> Union[Any, AzureUserAcrType0, Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _acr_type_0 = data
                acr_type_0: Union[Unset, AzureUserAcrType0]
                if isinstance(_acr_type_0, Unset):
                    acr_type_0 = UNSET
                else:
                    acr_type_0 = AzureUserAcrType0(_acr_type_0)

                return acr_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Any, AzureUserAcrType0, Unset], data)

        acr = _parse_acr(d.pop("acr", UNSET))

        amr = cast(List[str], d.pop("amr", UNSET))

        def _parse_appid(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        appid = _parse_appid(d.pop("appid", UNSET))

        def _parse_appidacr(data: object) -> Union[Any, AzureUserAppidacrType0, Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _appidacr_type_0 = data
                appidacr_type_0: Union[Unset, AzureUserAppidacrType0]
                if isinstance(_appidacr_type_0, Unset):
                    appidacr_type_0 = UNSET
                else:
                    appidacr_type_0 = AzureUserAppidacrType0(_appidacr_type_0)

                return appidacr_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Any, AzureUserAppidacrType0, Unset], data)

        appidacr = _parse_appidacr(d.pop("appidacr", UNSET))

        def _parse_unique_name(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        unique_name = _parse_unique_name(d.pop("unique_name", UNSET))

        def _parse_azp(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        azp = _parse_azp(d.pop("azp", UNSET))

        def _parse_azpacr(data: object) -> Union[Any, AzureUserAzpacrType0, Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                _azpacr_type_0 = data
                azpacr_type_0: Union[Unset, AzureUserAzpacrType0]
                if isinstance(_azpacr_type_0, Unset):
                    azpacr_type_0 = UNSET
                else:
                    azpacr_type_0 = AzureUserAzpacrType0(_azpacr_type_0)

                return azpacr_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Any, AzureUserAzpacrType0, Unset], data)

        azpacr = _parse_azpacr(d.pop("azpacr", UNSET))

        def _parse_preferred_username(data: object) -> Union[Any, Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, Unset, str], data)

        preferred_username = _parse_preferred_username(
            d.pop("preferred_username", UNSET)
        )

        is_guest = d.pop("is_guest", UNSET)

        azure_user = cls(
            aud=aud,
            iss=iss,
            iat=iat,
            nbf=nbf,
            exp=exp,
            sub=sub,
            ver=ver,
            claims=claims,
            access_token=access_token,
            idp=idp,
            aio=aio,
            name=name,
            scp=scp,
            roles=roles,
            wids=wids,
            groups=groups,
            oid=oid,
            tid=tid,
            uti=uti,
            rh=rh,
            acct=acct,
            auth_time=auth_time,
            ctry=ctry,
            email=email,
            family_name=family_name,
            fwd=fwd,
            given_name=given_name,
            idtyp=idtyp,
            in_corp=in_corp,
            ipaddr=ipaddr,
            login_hint=login_hint,
            onprem_sid=onprem_sid,
            pwd_exp=pwd_exp,
            pwd_url=pwd_url,
            sid=sid,
            tenant_ctry=tenant_ctry,
            tenant_region_scope=tenant_region_scope,
            upn=upn,
            verified_primary_email=verified_primary_email,
            verified_secondary_email=verified_secondary_email,
            vnet=vnet,
            xms_pdl=xms_pdl,
            xms_pl=xms_pl,
            xms_tpl=xms_tpl,
            ztdid=ztdid,
            acr=acr,
            amr=amr,
            appid=appid,
            appidacr=appidacr,
            unique_name=unique_name,
            azp=azp,
            azpacr=azpacr,
            preferred_username=preferred_username,
            is_guest=is_guest,
        )

        azure_user.additional_properties = d
        return azure_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ReadyResponse")


@_attrs_define
class ReadyResponse:
    """Define ready response model.

    Attributes:
        status (str): Strings are accepted as-is, int float and Decimal are
            coerced using str(v), bytes and bytearray are converted using
            v.decode(), enums inheriting from str are converted using
            v.value, and all other types cause an error.

    Raises:
        pydantic.error_wrappers.ValidationError: If any of provided attribute
            doesn't pass type validation.

        Attributes:
            status (str):
    """

    status: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = d.pop("status")

        ready_response = cls(
            status=status,
        )

        ready_response.additional_properties = d
        return ready_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
